''
'' This code is part of https://developer.mescius.com/document-solutions/dot-net-imaging-api/demos.
'' Copyright (c) MESCIUS inc. All rights reserved.
''
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Threading
Imports System.Reflection
Imports System.IO
Imports System.Linq
Imports System.Drawing
Imports System.Runtime.CompilerServices
Imports System.Runtime.InteropServices

Imports GrapeCity.Documents.Imaging
Imports GrapeCity.Documents.Drawing
Imports GrapeCity.Documents.Html

Public Module Util

    Private s_gen As Integer = 0
    Private s_fixedTime As DateTime = New DateTime(1989, 6, 21, 1, 23, 45, DateTimeKind.Utc)

    Public Class MimeTypes
        Public Const JPEG As String = "image/jpeg"
        Public Const PNG As String = "image/png"
        Public Const TIFF As String = "image/tiff"
        Public Const GIF As String = "image/gif"
        Public Const BMP As String = "image/bmp"
        Public Const ICO As String = "image/x-icon"
        Public Const SVG As String = "image/svg+xml"
        Public Const WEBP As String = "image/webp"
    End Class

    Public Property NoRandom As Boolean = False

    Public Function NewRandom() As Random
        If NoRandom Then
            Return New Random(0)
        Else
            Return New Random(Convert.ToInt32(DateTime.Now.Ticks And Int32.MaxValue) + Interlocked.Increment(s_gen))
        End If
    End Function

    Public Function TimeNow() As DateTime
        If NoRandom Then
            Return s_fixedTime
        Else
            Return DateTime.Now
        End If
    End Function

    Private s_browserBasePath As String = Nothing
    Private ReadOnly s_browserBasePath_lock As New Object()

    '' Set this to use the browser from a known location
    Public Sub SetBrowserBasePath(ByVal browserBasePath As String)
        SyncLock s_browserBasePath_lock
            s_browserBasePath = browserBasePath
        End SyncLock
    End Sub

    '' DsHtml v6.0+: this method provides a single target location to be used
    '' by the online samples, and also by all samples downloaded by a user,
    '' so that multiple samples can share the same browser installation
    '' rather than each sample downloading and installing its own copy.
    '' On a system with Chrome installed, the BrowserFetcher.GetSystemChromePath()
    '' call ensures that the installed browser is used.
    Public Function GetBrowserPath() As String
        If s_browserBasePath Is Nothing Then
            Dim systemPath = BrowserFetcher.GetSystemChromePath()
            If systemPath IsNot Nothing Then
                Return systemPath
            End If
        End If
        Dim pth As String
        If s_browserBasePath Is Nothing Then
            pth = Path.Combine(Path.GetTempPath(), ".gc-chromium")
        Else
            pth = s_browserBasePath
        End If
        Dim bf = New BrowserFetcher() With {.DestinationFolder = pth}
        Return bf.GetDownloadedPath()
    End Function

    '' DsHtml v6.0
    Public Function NewHtmlBrowser() As GcHtmlBrowser
        Dim pth = GetBrowserPath()
        Try
            If (RuntimeInformation.IsOSPlatform(OSPlatform.Linux)) Then
                Return New GcHtmlBrowser(pth, New LaunchOptions With {.RunWithNoSandbox = True})
            Else
                Return New GcHtmlBrowser(pth)
            End If
        Catch ex As Exception
            If pth = BrowserFetcher.GetSystemChromePath() Then
                Throw
            Else
                Throw New Exception("Exception occurred when launching the browser (see inner exception for details), " +
                    $"the browser installation in {Path.GetDirectoryName(pth)} may be corrupt. Please remove it and try again.", ex)
            End If
        End Try
    End Function

    '' https://stackoverflow.com/questions/4286487/is-there-any-lorem-ipsum-generator-in-c
    Public Function LoremIpsum(
            Optional ByVal numParagraphs As Integer = 5,
            Optional ByVal minSentences As Integer = 5,
            Optional ByVal maxSentences As Integer = 10,
            Optional ByVal minWords As Integer = 5,
            Optional ByVal maxWords As Integer = 30) As String

        Dim words =
            {
                "lorem", "ipsum", "dolor", "sit", "amet", "consectetur", "adipiscing", "elit", "mauris", "id", "volutpat",
                "tellus", "ullamcorper", "sem", "praesent", "et", "ante", "laoreet", "lobortis", "nunc", "congue",
                "nisi", "donec", "ac", "tempus", "sed", "feugiat", "pulvinar", "pharetra", "turpis", "nonummy",
                "at", "felis", "eget", "molestie", "euismod", "non", "aliquet", "diam", "proin", "mi", "nibh",
                "massa", "tincidunt", "ut", "dolore", "magna", "aliquam", "erat"
            }

        Dim commaMax = 10
        Dim rnd = NewRandom()
        Dim numSentences As Integer = rnd.Next(maxSentences - minSentences) + minSentences + 1
        Dim numWords As Integer = rnd.Next(maxWords - minWords) + minWords + 1

        Dim result = New StringBuilder()
        For p = 1 To numParagraphs
            For s = 1 To numSentences
                For w = 1 To numWords
                    Dim word As String = words(rnd.Next(words.Length))
                    If w = 1 Then
                        result.Append(Char.ToUpper(word(0)))
                        result.Append(word.Substring(1))
                    Else
                        If rnd.Next(commaMax) = 0 Then
                            result.Append(","c)
                        End If
                        result.Append(" "c)
                        result.Append(word)
                    End If
                Next
                result.Append(". ")
            Next
            result.Append(vbCrLf)
        Next
        Return result.ToString()
    End Function

    '' https://stackoverflow.com/questions/273313/randomize-a-listt
    <Extension()>
    Public Sub Shuffle(Of t)(ByVal list As IList(Of t))
        Dim rnd = NewRandom()
        Dim n = list.Count
        While (n > 1)
            n -= 1
            Dim k = rnd.Next(n + 1)
            Dim value = list(k)
            list(k) = list(n)
            list(n) = value
        End While
    End Sub

    ''' <summary>
    ''' Adds a note to the document - a text fragment on a colored background.
    ''' Used to insert meta info into sample documents.
    ''' </summary>
    ''' <param name="text">The text to draw.</param>
    ''' <param name="page">The page to draw on.</param>
    ''' <param name="bounds">The text bounds. If null, whole page with 1" margins all around is used.</param>
    ''' <returns>The actual rectangle occupied by the note.</returns>
    Public Function AddNote(ByVal text As String, ByVal g As GcBitmapGraphics, Optional ByVal bounds As Nullable(Of RectangleF) = Nothing) As RectangleF

        Dim tl = g.CreateTextLayout()
        Dim pad = g.Resolution / 8.0F
        '' tl.DefaultFormat.Font = StandardFonts.Helvetica
        tl.DefaultFormat.FontSize = 12
        If Not bounds.HasValue Then
            Dim margin = g.Resolution ' 1" margins by default
            bounds = New RectangleF(margin, margin, g.Width - margin * 2, g.Height - margin * 2)
        End If
        tl.MaxWidth = bounds.Value.Width
        tl.MaxHeight = bounds.Value.Height
        tl.MarginLeft = pad
        tl.MarginRight = pad
        tl.MarginTop = pad
        tl.MarginBottom = pad
        tl.Append(text)
        tl.PerformLayout(True)

        Dim rect = tl.ContentRectangle
        rect.Offset(bounds.Value.Location)
        rect.Inflate(pad, pad)

        g.FillRectangle(rect, Color.FromArgb(213, 240, 221))
        g.DrawRectangle(rect, Color.FromArgb(59, 170, 92), 0.5F)
        g.DrawTextLayout(tl, bounds.Value.Location)

        Return rect
    End Function

    ''' <summary>
    ''' Loads an image from a disk file.
    ''' <para>
    ''' The only point of this method Is to provide better performance on a Windows system,
    ''' by using GrapeCity.Documents.Drawing.Windows.WicImage which works via the
    ''' Windows Imaging Component framework. If WicImage Is Not supported, this method
    ''' defaults to platform-independent Image.FromFile() method.
    ''' </para>
    ''' </summary>
    ''' <param name="path">The image path.</param>
    ''' <returns>The Image object.</returns>
    Public Function ImageFromFile(ByVal path As String) As IImage
        If (Windows.GcWicBitmap.IsSupported) Then
            Return New Windows.GcWicBitmap(path)
        Else
            Return New GcBitmap(path)
        End If
    End Function
End Module

